@@ -1,3 +1,5 @@
+use std::path::PathBuf;
+
 use chrono::NaiveDateTime;
 use serde::Serialize;
 
@@ -15,6 +17,18 @@ pub struct File {
     pub mtime: Option<NaiveDateTime>,
 }
 
+// todo - remove, try_from the actual model, impl queryable...
+#[derive(Serialize, Clone, Debug)]
+pub struct OutFile {
+    pub id: i32,
+    pub hash: Hash,
+    pub path: PathBuf,
+    pub valid: bool,
+    pub added: NaiveDateTime,
+    pub size: i64,
+    pub mtime: Option<NaiveDateTime>,
+}
+
 #[derive(Insertable, Debug)]
 #[table_name = "files"]
 pub struct NewFile {
