@@ -1,7 +1,6 @@
 macro_rules! upend_insert_val {
     ($db_connection:expr, $entity:expr, $attribute:expr, $value:expr) => {{
-        insert_entry(
-            $db_connection,
+        $db_connection.insert_entry(
             Entry {
                 entity: $entity.clone(),
                 attribute: String::from($attribute),
@@ -13,8 +12,7 @@ macro_rules! upend_insert_val {
 
 macro_rules! upend_insert_addr {
     ($db_connection:expr, $entity:expr, $attribute:expr, $addr:expr) => {{
-        insert_entry(
-            $db_connection,
+        $db_connection.insert_entry(
             Entry {
                 entity: $entity.clone(),
                 attribute: String::from($attribute),
