@@ -1,3 +1,4 @@
+use std::borrow::Borrow;
 use std::convert::TryFrom;
 use std::path::{Component, Path, PathBuf};
 use std::sync::{Arc, Mutex, RwLock};
@@ -9,17 +10,13 @@ use crate::database::constants::{
     HIER_HAS_ATTR, IS_OF_TYPE_ATTR, TYPE_ADDR, TYPE_BASE_ATTR, TYPE_HAS_ATTR,
 };
 use crate::database::entry::{Entry, EntryValue, InvariantEntry};
-use crate::database::hierarchies::{resolve_path_cached, ResolveCache, UNode, UHierPath};
+use crate::database::hierarchies::{resolve_path_cached, ResolveCache, UHierPath, UNode};
 use crate::database::inner::models;
-use crate::database::{
-    file_set_valid, file_update_mtime, insert_entry, insert_file, retrieve_all_files, DbPool,
-    UPEND_SUBDIR,
-};
+use crate::database::{UpEndConnection, UpEndDatabase, UPEND_SUBDIR};
 use crate::util::hash::{Hash, Hashable};
 use crate::util::jobs::{Job, JobContainer, JobId, State};
 use anyhow::{Error, Result};
 use chrono::prelude::*;
-use diesel::Connection;
 use log::{debug, error, info, warn};
 use lru::LruCache;
 use rayon::prelude::*;
@@ -40,22 +37,18 @@ lazy_static! {
     static ref BLOB_TYPE_ADDR: Address = BLOB_TYPE_INVARIANT.entity().unwrap();
 }
 
-fn initialize_types(pool: &DbPool) -> Result<()> {
+fn initialize_types(connection: &UpEndConnection) -> Result<()> {
     // BLOB_TYPE
-    insert_entry(&pool.get()?, Entry::try_from(&*BLOB_TYPE_INVARIANT)?)?;
-    upend_insert_addr!(&pool.get()?, BLOB_TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
-    upend_insert_val!(&pool.get()?, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MTIME_KEY);
-    upend_insert_val!(&pool.get()?, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_SIZE_KEY);
-    upend_insert_val!(&pool.get()?, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MIME_KEY);
+    connection.insert_entry(Entry::try_from(&*BLOB_TYPE_INVARIANT)?)?;
+    upend_insert_addr!(connection, BLOB_TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MTIME_KEY);
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_SIZE_KEY);
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MIME_KEY);
 
     Ok(())
 }
 
-pub async fn rescan_vault(
-    pool: DbPool,
-    directory: PathBuf,
-    job_container: Arc<RwLock<JobContainer>>,
-) {
+pub async fn rescan_vault(db: Arc<UpEndDatabase>, job_container: Arc<RwLock<JobContainer>>) {
     let job_id = job_container
         .write()
         .unwrap()
@@ -64,8 +57,7 @@ pub async fn rescan_vault(
 
     let job_container_rescan = job_container.clone();
     let result =
-        actix_web::web::block(move || _rescan_vault(pool, directory, job_container_rescan, job_id))
-            .await;
+        actix_web::web::block(move || _rescan_vault(db, job_container_rescan, job_id)).await;
 
     if result.is_err() {
         let err = result.err().unwrap();
@@ -78,18 +70,12 @@ pub async fn rescan_vault(
             .unwrap();
     }
 }
-struct PragmaSynchronousGuard<'a>(&'a DbPool);
+struct PragmaSynchronousGuard<'a>(&'a UpEndConnection);
 
 impl Drop for PragmaSynchronousGuard<'_> {
     fn drop(&mut self) {
         debug!("Re-enabling synchronous mode.");
-        let connection = self.0.get();
-        let res: Result<_, String> = match connection {
-            Ok(connection) => connection
-                .execute("PRAGMA synchronous = NORMAL;")
-                .map_err(|err| format!("{}", err)),
-            Err(err) => Err(format!("{}", err)),
-        };
+        let res = self.0.execute("PRAGMA synchronous = NORMAL;");
         if let Err(err) = res {
             error!(
                 "Error setting synchronous mode back to NORMAL! Data loss possible! {}",
@@ -109,28 +95,30 @@ enum UpdatePathOutcome {
     Failed(PathBuf, Error),
 }
 
-fn _rescan_vault<T: AsRef<Path>>(
-    pool: DbPool,
-    directory: T,
+fn _rescan_vault<D: Borrow<UpEndDatabase>>(
+    db: D,
     job_container: Arc<RwLock<JobContainer>>,
     job_id: JobId,
 ) -> Result<Vec<UpdatePathOutcome>> {
     let start = Instant::now();
     info!("Vault rescan started.");
 
+    let db = db.borrow();
+    let connection = db.connection()?;
+
     // Initialize types, etc...
     debug!("Initializing DB types.");
-    initialize_types(&pool)?;
+    initialize_types(&connection)?;
 
     // Disable syncing in SQLite for the duration of the import
     debug!("Disabling SQLite synchronous mode");
-    pool.get()?.execute("PRAGMA synchronous = OFF;")?;
-    let _guard = PragmaSynchronousGuard(&pool);
+    connection.execute("PRAGMA synchronous = OFF;")?;
+    let _guard = PragmaSynchronousGuard(&connection);
 
     // Walk through the vault, find all paths
     debug!("Traversing vault directory");
-    let absolute_dir_path = fs::canonicalize(&directory)?;
-    let path_entries: Vec<PathBuf> = WalkDir::new(&directory)
+    let absolute_dir_path = fs::canonicalize(&*db.vault_path)?;
+    let path_entries: Vec<PathBuf> = WalkDir::new(&*db.vault_path)
         .follow_links(true)
         .into_iter()
         .filter_map(|e| e.ok())
@@ -140,8 +128,7 @@ fn _rescan_vault<T: AsRef<Path>>(
         .collect();
 
     // Prepare for processing
-    let rw_pool = Arc::new(RwLock::new(pool.clone()));
-    let existing_files = Arc::new(RwLock::new(retrieve_all_files(&pool.get()?)?));
+    let existing_files = Arc::new(RwLock::new(connection.retrieve_all_files()?));
 
     // Actual processing
     let count = RwLock::new(0_usize);
@@ -151,7 +138,7 @@ fn _rescan_vault<T: AsRef<Path>>(
         .into_par_iter()
         .map(|path| {
             let result = _process_directory_entry(
-                &rw_pool,
+                db.connection().unwrap(),
                 &resolve_cache,
                 path.clone(),
                 &absolute_dir_path,
@@ -178,10 +165,9 @@ fn _rescan_vault<T: AsRef<Path>>(
 
     let existing_files = existing_files.read().unwrap();
 
-    let connection = pool.get()?;
     let cleanup_results = existing_files.iter().filter(|f| f.valid).map(|file| {
         let trans_result = connection.transaction::<_, Error, _>(|| {
-            file_set_valid(&connection, file.id, false)?;
+            connection.file_set_valid(file.id, false)?;
             // remove_object(&connection, )?
             Ok(())
         });
@@ -225,8 +211,8 @@ fn _rescan_vault<T: AsRef<Path>>(
     drop(_guard);
 
     info!(
-        "Finished updating {} ({} created, {} deleted, {} left unchanged). Took {}s.",
-        directory.as_ref().display(),
+        "Finished updating {:?} ({} created, {} deleted, {} left unchanged). Took {}s.",
+        db.vault_path,
         created,
         deleted,
         unchanged,
@@ -237,7 +223,7 @@ fn _rescan_vault<T: AsRef<Path>>(
 }
 
 fn _process_directory_entry<P: AsRef<Path>>(
-    db_pool: &Arc<RwLock<DbPool>>,
+    connection: UpEndConnection,
     resolve_cache: &Arc<Mutex<ResolveCache>>,
     path: PathBuf,
     directory_path: &P,
@@ -246,7 +232,6 @@ fn _process_directory_entry<P: AsRef<Path>>(
     debug!("Processing: {:?}", path);
 
     // Prepare the data
-    let connection = &db_pool.write().unwrap().get()?;
     let existing_files = Arc::clone(existing_files);
 
     let normalized_path = path.strip_prefix(&directory_path)?;
@@ -288,11 +273,11 @@ fn _process_directory_entry<P: AsRef<Path>>(
 
             if same_mtime || same_hash {
                 if mtime != existing_file.mtime {
-                    file_update_mtime(connection, existing_file.id, mtime)?;
+                    connection.file_update_mtime(existing_file.id, mtime)?;
                 }
 
                 if !existing_file.valid {
-                    file_set_valid(connection, existing_file.id, true)?;
+                    connection.file_set_valid(existing_file.id, true)?;
                 }
 
                 let mut existing_files_write = existing_files.write().unwrap();
@@ -340,11 +325,11 @@ fn _process_directory_entry<P: AsRef<Path>>(
         value: EntryValue::Value(Value::from(size)),
     };
 
-    let mime_entry = Entry {
+    let mime_entry = tree_magic_mini::from_filepath(&path).map(|mime_type| Entry {
         entity: Address::Hash(file_hash.clone()),
         attribute: FILE_MIME_KEY.to_string(),
-        value: EntryValue::Value(Value::String(tree_magic::from_filepath(&path))),
-    };
+        value: EntryValue::Value(Value::String(mime_type.to_string())),
+    });
 
     // Finally, add the appropriate entries w/r/t virtual filesystem location
     let components = normalized_path.components().collect::<Vec<Component>>();
@@ -357,22 +342,24 @@ fn _process_directory_entry<P: AsRef<Path>>(
             }))
             .collect(),
     );
-    let resolved_path = resolve_path_cached(connection, &upath, true, resolve_cache)?;
+    let resolved_path = resolve_path_cached(&connection, &upath, true, resolve_cache)?;
     let parent_dir = resolved_path.last().unwrap();
 
     connection.transaction::<_, Error, _>(|| {
-        insert_file(connection, new_file)?;
+        connection.insert_file(new_file)?;
 
-        insert_entry(connection, type_entry)?;
-        insert_entry(connection, size_entry)?;
-        insert_entry(connection, mime_entry)?;
+        connection.insert_entry(type_entry)?;
+        connection.insert_entry(size_entry)?;
+        if let Some(mime_entry) = mime_entry {
+            connection.insert_entry(mime_entry)?;
+        }
 
         let dir_has_entry = Entry {
             entity: parent_dir.clone(),
             attribute: HIER_HAS_ATTR.to_string(),
             value: EntryValue::Address(Address::Hash(file_hash.clone())),
         };
-        let dir_has_entry_addr = insert_entry(connection, dir_has_entry)?;
+        let dir_has_entry_addr = connection.insert_entry(dir_has_entry)?;
 
         let name_entry = Entry {
             entity: dir_has_entry_addr,
@@ -381,7 +368,7 @@ fn _process_directory_entry<P: AsRef<Path>>(
                 filename.as_os_str().to_string_lossy().to_string(),
             )),
         };
-        insert_entry(connection, name_entry)?;
+        connection.insert_entry(name_entry)?;
 
         info!("Added: {:?}", path);
         Ok(UpdatePathOutcome::Added(path.clone()))
@@ -390,18 +377,18 @@ fn _process_directory_entry<P: AsRef<Path>>(
 
 #[cfg(test)]
 mod test {
-    use crate::database::open_upend;
+    use crate::database::UpEndDatabase;
     use crate::util;
 
     use super::*;
     use std::fs::File;
     use std::io::Write;
-    use tempdir::TempDir;
+    use tempfile::TempDir;
 
     #[test]
     fn test_rescan() {
         // Prepare temporary filesystem structure
-        let temp_dir = TempDir::new("upend-test").unwrap();
+        let temp_dir = TempDir::new().unwrap();
 
         let file_path = temp_dir.path().join("my-temporary-note.txt");
         let mut tmp_file = File::create(file_path).unwrap();
@@ -416,7 +403,7 @@ mod test {
 
         // Initialize database
 
-        let open_result = open_upend(&temp_dir, None, true).unwrap();
+        let open_result = UpEndDatabase::open(&temp_dir, None, true).unwrap();
         let job_container = Arc::new(RwLock::new(util::jobs::JobContainer::default()));
         let job_id = job_container
             .write()
@@ -425,12 +412,7 @@ mod test {
             .unwrap();
 
         // Initial scan
-        let rescan_result = _rescan_vault(
-            open_result.pool.clone(),
-            temp_dir.as_ref().to_path_buf(),
-            job_container.clone(),
-            job_id,
-        );
+        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
@@ -441,12 +423,7 @@ mod test {
 
         // Modification-less rescan
 
-        let rescan_result = _rescan_vault(
-            open_result.pool.clone(),
-            temp_dir.as_ref().to_path_buf(),
-            job_container.clone(),
-            job_id,
-        );
+        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
@@ -459,12 +436,7 @@ mod test {
 
         std::fs::remove_file(temp_dir.path().join("hello-world.txt")).unwrap();
 
-        let rescan_result = _rescan_vault(
-            open_result.pool,
-            temp_dir.as_ref().to_path_buf(),
-            job_container,
-            job_id,
-        );
+        let rescan_result = _rescan_vault(&open_result.db, job_container, job_id);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
