@@ -15,12 +15,17 @@ use clap::{App as ClapApp, Arg};
 use log::{info, warn};
 use std::sync::{Arc, RwLock};
 
+use crate::database::UpEndDatabase;
+
 mod addressing;
 mod database;
 mod filesystem;
 mod routes;
 mod util;
 
+#[cfg(feature = "previews")]
+mod previews;
+
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 fn main() -> Result<()> {
@@ -85,23 +90,32 @@ fn main() -> Result<()> {
 
     let vault_path = PathBuf::from(matches.value_of("DIRECTORY").unwrap());
 
-    let open_result = database::open_upend(
+    let open_result = UpEndDatabase::open(
         &vault_path,
         matches.value_of("DB_PATH").map(PathBuf::from),
         matches.is_present("REINITIALIZE"),
     )
     .expect("failed to open database!");
 
-    let db_pool = open_result.pool;
+    let upend = Arc::new(open_result.db);
 
-    let bind: SocketAddr = matches
+    #[cfg(feature = "previews")]
+    let preview_store = Some(Arc::new(crate::previews::PreviewStore::new(
+        upend.db_path.join("previews"),
+        upend.clone(),
+    )));
+
+    #[cfg(not(feature = "previews"))]
+    let preview_store = None;
+
+    let mut bind: SocketAddr = matches
         .value_of("BIND")
         .unwrap()
         .parse()
         .expect("Incorrect bind format.");
-    info!("Starting server at: {}", &bind);
 
     let state = routes::State {
+        upend: upend.clone(),
         vault_name: Some(
             matches
                 .value_of("VAULT_NAME")
@@ -115,9 +129,8 @@ fn main() -> Result<()> {
                         .into_owned()
                 }),
         ),
-        directory: vault_path.clone(),
-        db_pool: db_pool.clone(),
         job_container: job_container.clone(),
+        preview_store,
     };
 
     // Start HTTP server
@@ -126,35 +139,58 @@ fn main() -> Result<()> {
         warn!("No Web UI directory present ({:?}), disabling...", ui_path);
     }
     let ui_enabled = ui_path.exists() && !matches.is_present("NO_UI");
-    HttpServer::new(move || {
-        let app = App::new()
-            .data(state.clone())
-            .wrap(middleware::Logger::default().exclude("/api/jobs"))
-            .service(routes::get_raw)
-            .service(routes::get_query)
-            .service(routes::get_object)
-            .service(routes::put_object)
-            .service(routes::delete_object)
-            .service(routes::api_refresh)
-            .service(routes::list_hier)
-            .service(routes::list_hier_roots)
-            .service(routes::latest_files)
-            .service(routes::get_file)
-            .service(routes::get_jobs)
-            .service(routes::get_info);
 
-        if ui_enabled {
-            app.service(actix_files::Files::new("/", &ui_path).index_file("index.html"))
+    let mut cnt = 0;
+    let server = loop {
+        let state = state.clone();
+        let ui_path = ui_path.clone();
+
+        let server = HttpServer::new(move || {
+            let app = App::new()
+                .data(state.clone())
+                .wrap(middleware::Logger::default().exclude("/api/jobs"))
+                .service(routes::get_raw)
+                .service(routes::get_thumbnail)
+                .service(routes::get_query)
+                .service(routes::get_object)
+                .service(routes::put_object)
+                .service(routes::delete_object)
+                .service(routes::api_refresh)
+                .service(routes::list_hier)
+                .service(routes::list_hier_roots)
+                .service(routes::latest_files)
+                .service(routes::get_file)
+                .service(routes::get_jobs)
+                .service(routes::get_info);
+
+            if ui_enabled {
+                app.service(actix_files::Files::new("/", &ui_path).index_file("index.html"))
+            } else {
+                app
+            }
+        });
+
+        let bind_result = server.bind(&bind);
+        if let Ok(server) = bind_result {
+            break server;
         } else {
-            app
+            warn!("Failed to bind at {:?}, trying next port number...", bind);
+            bind.set_port(bind.port() + 1);
         }
-    })
-    .bind(&bind)?
-    .run();
+
+        if cnt > 10 {
+            panic!("Couldn't start server.")
+        } else {
+            cnt += 1;
+        }
+    };
+
+    info!("Starting server at: {}", &bind);
+    server.run();
 
     if !matches.is_present("NO_INITIAL_UPDATE") {
         info!("Running initial update...");
-        actix::spawn(filesystem::rescan_vault(db_pool, vault_path, job_container));
+        actix::spawn(filesystem::rescan_vault(upend, job_container));
     }
 
     #[cfg(feature = "desktop")]
