@@ -2,13 +2,13 @@ use crate::addressing::{Address, Addressable};
 use crate::database::entry::{Entry, InEntry};
 use crate::database::hierarchies::{list_roots, resolve_path, UHierPath};
 use crate::database::lang::Query;
-use crate::database::{
-    get_latest_files, insert_entry, query, remove_object, retrieve_file, retrieve_object, DbPool,
-};
+use crate::database::UpEndDatabase;
+use crate::previews::PreviewStore;
 use crate::util::hash::{decode, encode};
 use crate::util::jobs::JobContainer;
 use actix_files::NamedFile;
 use actix_web::error::{ErrorBadRequest, ErrorInternalServerError, ErrorNotFound};
+use actix_web::http;
 use actix_web::{delete, error, get, post, put, web, Either, Error, HttpResponse};
 use anyhow::Result;
 use futures_util::StreamExt;
@@ -17,7 +17,6 @@ use serde::Deserialize;
 use serde_json::json;
 use std::collections::HashMap;
 use std::convert::TryFrom;
-use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 
 #[cfg(feature = "desktop")]
@@ -27,10 +26,10 @@ const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[derive(Clone)]
 pub struct State {
+    pub upend: Arc<UpEndDatabase>,
     pub vault_name: Option<String>,
-    pub directory: PathBuf,
-    pub db_pool: DbPool,
     pub job_container: Arc<RwLock<JobContainer>>,
+    pub preview_store: Option<Arc<PreviewStore>>,
 }
 
 #[derive(Deserialize)]
@@ -47,33 +46,56 @@ pub async fn get_raw(
     let address = Address::decode(&decode(hash.into_inner()).map_err(ErrorInternalServerError)?)
         .map_err(ErrorInternalServerError)?;
     if let Address::Hash(hash) = address {
-        let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-        let files = retrieve_file(&connection, hash).map_err(ErrorInternalServerError)?;
+        let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+        let files = connection
+            .retrieve_file(hash)
+            .map_err(ErrorInternalServerError)?;
         if let Some(file) = files.get(0) {
-            let file_path = state.directory.join(&file.path);
+            let file_path = state.upend.vault_path.join(&file.path);
 
-            if !query.native.is_some() {
+            if query.native.is_none() {
                 Ok(Either::A(NamedFile::open(file_path)?))
-            } else {
-                if cfg!(feature = "desktop") {
-                    #[cfg(feature = "desktop")]
-                    {
-                        info!("Opening {:?}...", file_path);
-                        if !file_path.is_executable() {
-                            opener::open(file_path).map_err(error::ErrorServiceUnavailable)?;
-                            return Ok(Either::B(HttpResponse::NoContent().finish()));
-                        }
-                    }
-                    Err(error::ErrorForbidden(""))
-                } else {
-                    Err(error::ErrorBadRequest("Desktop features not enabled."))
+            } else if cfg!(feature = "desktop") {
+                #[cfg(feature = "desktop")]
+                {
+                    info!("Opening {:?}...", file_path);
+                    let mut response = HttpResponse::NoContent();
+                    let path = if !file_path.is_executable() {
+                        file_path
+                    } else {
+                        response
+                            .header(
+                                http::header::WARNING,
+                                "199 - Opening parent directory due to file being executable.",
+                            )
+                            .header(
+                                http::header::ACCESS_CONTROL_EXPOSE_HEADERS,
+                                http::header::WARNING.to_string(),
+                            );
+
+                        file_path
+                            .parent()
+                            .ok_or_else(|| {
+                                ErrorInternalServerError("No parent to open as fallback.")
+                            })?
+                            .to_path_buf()
+                    };
+                    opener::open(path).map_err(error::ErrorServiceUnavailable)?;
+                    Ok(Either::B(response.finish()))
                 }
+
+                #[cfg(not(feature = "desktop"))]
+                unreachable!()
+            } else {
+                Err(error::ErrorNotImplemented("Desktop features not enabled."))
             }
         } else {
             Err(error::ErrorNotFound("NOT FOUND"))
         }
     } else {
-        Err(ErrorBadRequest("Address does not refer to a file."))
+        Err(ErrorBadRequest(
+            "Address does not refer to a rawable object.",
+        ))
     }
 }
 
@@ -87,11 +109,12 @@ pub async fn get_query(
     state: web::Data<State>,
     web::Query(info): web::Query<QueryRequest>,
 ) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
 
-    let sexp = lexpr::from_str(info.query.as_str()).map_err(ErrorBadRequest)?;
-    let in_query = Query::try_from(&sexp).map_err(ErrorBadRequest)?;
-    let entries = query(&connection, in_query).map_err(ErrorInternalServerError)?;
+    let in_query: Query = info.query.as_str().parse().map_err(ErrorBadRequest)?;
+    let entries = connection
+        .query(in_query)
+        .map_err(ErrorInternalServerError)?;
     let mut result: HashMap<String, Entry> = HashMap::new();
     for entry in entries {
         result.insert(
@@ -129,13 +152,13 @@ pub async fn get_object(
     state: web::Data<State>,
     address_str: web::Path<String>,
 ) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-    let result: Vec<Entry> = retrieve_object(
-        &connection,
-        Address::decode(&decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
-            .map_err(ErrorBadRequest)?,
-    )
-    .map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+    let result: Vec<Entry> = connection
+        .retrieve_object(
+            Address::decode(&decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
+                .map_err(ErrorBadRequest)?,
+        )
+        .map_err(ErrorInternalServerError)?;
 
     debug!("{:?}", result);
     Ok(HttpResponse::Ok().json(result.as_hash().map_err(ErrorInternalServerError)?))
@@ -148,7 +171,7 @@ pub async fn put_object(
     state: web::Data<State>,
     mut payload: web::Payload,
 ) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
 
     let mut body = web::BytesMut::new();
     while let Some(chunk) = payload.next().await {
@@ -163,8 +186,9 @@ pub async fn put_object(
     let in_entry = serde_json::from_slice::<InEntry>(&body).map_err(ErrorBadRequest)?;
     let entry = Entry::try_from(in_entry).map_err(ErrorInternalServerError)?;
 
-    let result_address =
-        insert_entry(&connection, entry.clone()).map_err(ErrorInternalServerError)?;
+    let result_address = connection
+        .insert_entry(entry.clone())
+        .map_err(ErrorInternalServerError)?;
 
     Ok(HttpResponse::Ok().json(
         [(
@@ -182,13 +206,13 @@ pub async fn delete_object(
     state: web::Data<State>,
     address_str: web::Path<String>,
 ) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-    let _ = remove_object(
-        &connection,
-        Address::decode(&decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
-            .map_err(ErrorInternalServerError)?,
-    )
-    .map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+    let _ = connection
+        .remove_object(
+            Address::decode(&decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
+                .map_err(ErrorInternalServerError)?,
+        )
+        .map_err(ErrorInternalServerError)?;
 
     Ok(HttpResponse::Ok().finish())
 }
@@ -198,7 +222,7 @@ pub async fn list_hier(
     state: web::Data<State>,
     path: web::Path<String>,
 ) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
     if path.is_empty() {
         Ok(HttpResponse::MovedPermanently()
             .header("Location", "/api/hier_roots")
@@ -220,12 +244,12 @@ pub async fn list_hier(
 
 #[get("/api/hier_roots")]
 pub async fn list_hier_roots(state: web::Data<State>) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
 
     let result = list_roots(&connection)
         .map_err(ErrorInternalServerError)?
         .into_iter()
-        .map(|root| retrieve_object(&connection, root))
+        .map(|root| connection.retrieve_object(root))
         .collect::<Result<Vec<Vec<Entry>>>>()
         .map_err(ErrorInternalServerError)?
         .concat();
@@ -235,11 +259,8 @@ pub async fn list_hier_roots(state: web::Data<State>) -> Result<HttpResponse, Er
 
 #[post("/api/refresh")]
 pub async fn api_refresh(state: web::Data<State>) -> Result<HttpResponse, Error> {
-    let _pool = state.db_pool.clone();
-    let _directory = state.directory.clone();
     actix::spawn(crate::filesystem::rescan_vault(
-        _pool,
-        _directory,
+        state.upend.clone(),
         state.job_container.clone(),
     ));
     Ok(HttpResponse::Ok().finish())
@@ -254,8 +275,10 @@ pub async fn get_file(
         .map_err(ErrorInternalServerError)?;
 
     if let Address::Hash(hash) = address {
-        let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-        let response = retrieve_file(&connection, hash).map_err(ErrorInternalServerError)?;
+        let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+        let response = connection
+            .retrieve_file(hash)
+            .map_err(ErrorInternalServerError)?;
 
         Ok(HttpResponse::Ok().json(response))
     } else {
@@ -265,8 +288,10 @@ pub async fn get_file(
 
 #[get("/api/files/latest")]
 pub async fn latest_files(state: web::Data<State>) -> Result<HttpResponse, Error> {
-    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-    let files = get_latest_files(&connection, 100).map_err(ErrorInternalServerError)?;
+    let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+    let files = connection
+        .get_latest_files(100)
+        .map_err(ErrorInternalServerError)?;
     Ok(HttpResponse::Ok().json(&files))
 }
 
@@ -280,7 +305,38 @@ pub async fn get_jobs(state: web::Data<State>) -> Result<HttpResponse, Error> {
 pub async fn get_info(state: web::Data<State>) -> Result<HttpResponse, Error> {
     Ok(HttpResponse::Ok().json(json!({
         "name": state.vault_name,
-        "location": state.directory,
+        "location": &*state.upend.vault_path,
         "version": VERSION
     })))
 }
+
+#[get("/api/thumb/{hash}")]
+pub async fn get_thumbnail(
+    state: web::Data<State>,
+    hash: web::Path<String>,
+) -> Result<NamedFile, Error> {
+    if let Some(preview_store) = &state.preview_store {
+        let address =
+            Address::decode(&decode(hash.into_inner()).map_err(ErrorInternalServerError)?)
+                .map_err(ErrorInternalServerError)?;
+        if let Address::Hash(hash) = address {
+            let preview_path = preview_store
+                .get(hash)
+                .map_err(error::ErrorInternalServerError)?;
+
+            let mut file = NamedFile::open(&preview_path)?.disable_content_disposition();
+            if let Some(mime_type) = tree_magic_mini::from_filepath(&preview_path) {
+                if let Ok(mime) = mime_type.parse() {
+                    file = file.set_content_type(mime);
+                }
+            }
+            Ok(file)
+        } else {
+            Err(ErrorBadRequest(
+                "Address does not refer to a previewable object.",
+            ))
+        }
+    } else {
+        Err(error::ErrorNotImplemented("Previews not enabled."))
+    }
+}
