@@ -13,12 +13,6 @@ export interface ListingResult {
 
 // export type OrderedListing = [Address, IEntry][];
 
-export interface Job {
-  title: string;
-  progress: number;
-  state: "InProgress" | "Done" | "Failed";
-}
-
 export interface IFile {
   hash: string;
   path: string;
@@ -28,6 +22,13 @@ export interface IFile {
   mtime: string;
 }
 
+export interface IJob {
+  title: string;
+  progress: number;
+  state: "InProgress" | "Done" | "Failed";
+}
+
+
 export interface VaultInfo {
   name: string | null;
   location: string;
