@@ -1,6 +1,6 @@
 // import { useSWR } from "sswr";
 import LRU from "lru-cache";
-import { derived, Readable, Writable, writable } from "svelte/store";
+import { derived, Readable } from "svelte/store";
 import { UpListing, UpObject } from "upend";
 import type { ListingResult } from "upend/types";
 import { useSWR } from "../util/fetch";
@@ -40,7 +40,6 @@ export function query(query: () => string) {
 
   return {
     result,
-    data,
     error,
     revalidate,
   };
